
//compile: csc test2.cs repcoder.cs

using System;

class App
{
 public static void ShowLastError()
   {
    if( RCALC.LastError() ) // Check for the last error (in the current thread)
      {
       RCALC.ShowError((IntPtr)0); // Show error in MessageBox
       Console.WriteLine(RCALC.ErrorString() + "\n"); // Print error to Console
      }
   }
//-----------------------------      

 public static void ShowResult(RCALC rc)
   {
    int nOut = rc.OutputCount();
    for(int i=1; i<=nOut; i++)
      {
       bool bNull = rc.OutIsNull(i); // Check for null result

       string name = rc.OutName(i);   
       string type_name = rc.OutTypeName(i);

       string result;
       if( bNull ) result = "<NULL>";
       else result = rc.OutString(i); // Get the result as string

       Console.WriteLine(i + "). " + name + " [" + type_name + "]: " + result);
      }   
   }
//-----------------------------

 public static void Main()
   {
// Init repcoder
    if( !Repcoder.init() ) 
      {
       Console.WriteLine("Repcoder.init() error ...");
       return;
      }

// Create RCALC object
    RCALC rc = new RCALC();
    
// Compile RCALC script file
    bool ok = rc.CompileFile(".\\Samples\\test2.txt", 0);
    if( !ok )
      {
       RCALC.ShowError((IntPtr)0);
       if( RCALC.LastError() ) Console.WriteLine(RCALC.ErrorString() + "\n");
       
       Repcoder.exit();
       return;
      }
           
// Show compiled text
    string compstr = rc.CompiledText(true);
    if( compstr != String.Empty ) 
      { 
       Console.WriteLine("\n******* Compiled text ***************\n");
       Console.WriteLine(compstr);
       Console.WriteLine("\n\n******* End of compiled text ********\n");
      }

// Get and print the number of output values               
    int nOut = rc.OutputCount();
    Console.WriteLine("nOut = " + nOut + "\n");
    
// Get and print the output names and their types
    for(int i=1; i<=nOut; i++)
      {
       string name = rc.OutName(i);
       string type_name = rc.OutTypeName(i);
       Console.WriteLine(i + "). " + name + " [" + type_name + "]");
      }

    Console.WriteLine("\nResults:\n");
        
// Execute the script 7 times with different sets of input parameters

    ok = rc.Execute("-12.3456###-5.67"); // standard separator string: "###"
    if( !ok ) ShowLastError();
    ShowResult(rc);
    Console.WriteLine();
    
    RCALC rc2 = new RCALC(rc); // make a copy of rc (will be used at the end)

    ok = rc.Execute("0.23|8.43", "|"); // custom separator string: "|"
    if( !ok ) ShowLastError();
    ShowResult(rc);
    Console.WriteLine();

    ok = rc.Execute("45 @ -34.67", " @ "); // custom separator string: " @ "
    if( !ok ) ShowLastError();
    ShowResult(rc);
    Console.WriteLine();
    
    ok = rc.Execute("###457.93"); // NULL value passed as parameter 1
    if( !ok ) ShowLastError();
    ShowResult(rc);
    Console.WriteLine();

    ok = rc.Execute("786###"); // NULL value passed as parameter 2
    if( !ok ) ShowLastError();
    ShowResult(rc);
    Console.WriteLine();

    ok = rc.Execute(); // NULL values passed for all input parameters
    if( !ok ) ShowLastError();
    ShowResult(rc);
    Console.WriteLine();
    
// Execute passing parameters in the string table
    string[] stab = { "7.234", "-0.03498" };
    ok = rc.Execute(stab);
    if( !ok ) ShowLastError();
    ShowResult(rc);
    Console.WriteLine();
        
// Show some results for rc2 (copy of rc after the first execute)
Console.WriteLine();
Console.WriteLine("Out1 (numeric) = " + rc2.OutNumeric(1)); // convert to long (64-bit integer)
Console.WriteLine("Out1 (double) = " + rc2.OutDouble(1)); // convert to double
Console.WriteLine("Out1 (int) = " + rc2.OutInt(1)); // convert to int
Console.WriteLine("Out1 (string) = " + rc2.OutString(1)); // show as string

Console.WriteLine();
Console.WriteLine("Out2 (numeric) = " + rc2.OutNumeric(2));
Console.WriteLine("Out2 (double) = " + rc2.OutDouble(2));
Console.WriteLine("Out2 (int) = " + rc2.OutInt(2));
Console.WriteLine("Out2 (string) = " + rc2.OutString(2));

Console.WriteLine("");
Console.WriteLine("Out3 (numeric) = " + rc2.OutNumeric(3));
Console.WriteLine("Out3 (double) = " + rc2.OutDouble(3));
Console.WriteLine("Out3 (int) = " + rc2.OutInt(3));
Console.WriteLine("Out3 (string) = " + rc2.OutString(3));

long tm = rc2.OutTime(8); // date, time and timestamp are returned as long (64-bit)
Console.WriteLine();
Console.WriteLine("Out8 (date) = " + RCALC.Year(tm) + "-" + RCALC.Month(tm) + "-" + RCALC.Day(tm)); // extract date parameters from this long result
Console.WriteLine("Out8 (string) = " + rc2.OutString(8));

// Exit repcoder
    Repcoder.exit();
   }
}
