
#include "rcalc_class.h"

/***********************************************/

RCALC::RCALC()
{
Handle = rcalc_alloc_handle();
}
/*---------------------------------------*/

RCALC::RCALC(RCALC& rc)
{
Handle = rcalc_copy_handle(rc.Handle);
}
/*---------------------------------------*/

RCALC::RCALC(int OtherHandle)
{
Handle = rcalc_copy_handle(OtherHandle);
}
/*---------------------------------------*/

RCALC::~RCALC()
{
if( Handle > 0 ) rcalc_free_handle(Handle);
}
/*---------------------------------------*/

bool RCALC::CopyFrom(RCALC& rc)
{
if( Handle == rc.Handle ) return 0;

if( Handle > 0 )
  {
   int ok = rcalc_free_handle(Handle);
   if( ok )
     {
      Handle = rcalc_copy_handle(rc.Handle);
      return (Handle > 0);
     }
   else return 0;
  }
else return 0;
}
/*---------------------------------------*/

int RCALC::GetState()
{
if( Handle <= 0 ) return 0;
return rcalc_get_state(Handle);
}
/*---------------------------------------*/

bool RCALC::IsValid()
{
if( Handle <= 0 ) return 0;
return rcalc_is_valid(Handle);
}
/*---------------------------------------*/

bool RCALC::IsCompiled()
{
if( Handle <= 0 ) return 0;
return rcalc_is_compiled(Handle);
}
/*---------------------------------------*/

bool RCALC::IsExecuted()
{
if( Handle <= 0 ) return 0;
return rcalc_is_executed(Handle);
}
/*---------------------------------------*/

void RCALC::SetStatusWindow(HWND hStatWnd)
{
rcalc_set_status_window(Handle, hStatWnd);
}
/*---------------------------------------*/

bool RCALC::Compile(const wchar_t* pText)
{
if( Handle <= 0 || !pText ) return 0;
if( pText[0] == 0 ) return 0;
return rcalc_compile(Handle, pText);
}
/*---------------------------------------*/

bool RCALC::Compile(const char* pText)
{
if( Handle <= 0 || !pText ) return 0;
if( pText[0] == 0 ) return 0;
return rcalc_compile_a(Handle, pText);
}
/*---------------------------------------*/

bool RCALC::CompileFile(const wchar_t* FileName, int Format)
{
if( Handle <= 0 || !FileName ) return 0;
if( FileName[0] == 0 ) return 0;
return rcalc_compile_file(Handle, FileName, Format);
}
/*---------------------------------------*/

bool RCALC::CompileFile(const char* FileName, int Format)
{
if( Handle <= 0 || !FileName ) return 0;
if( FileName[0] == 0 ) return 0;
return rcalc_compile_file_a(Handle, FileName, Format);
}
/*---------------------------------------*/

bool RCALC::Execute(int nInput, const wchar_t* const* pInputValues)
{
if( Handle <= 0 ) return 0;
return rcalc_execute(Handle, nInput, pInputValues);
}
/*---------------------------------------*/

bool RCALC::Execute(int nInput, const char* const* pInputValues)
{
if( Handle <= 0 ) return 0;
return rcalc_execute_a(Handle, nInput, pInputValues);
}
/*---------------------------------------*/

bool RCALC::Execute(const wchar_t* InputValues, const wchar_t* Separator)
{
if( Handle <= 0 ) return 0;
return rcalc_execute2(Handle, InputValues, Separator);
}
/*---------------------------------------*/

bool RCALC::Execute(const char* InputValues, const char* Separator)
{
if( Handle <= 0 ) return 0;
return rcalc_execute2_a(Handle, InputValues, Separator);
}
/*---------------------------------------*/

bool RCALC::Execute(int nInput, const std::wstring* pInputValues)
{
if( Handle <= 0 ) return 0;
if( nInput <= 0 || !pInputValues ) return rcalc_execute(Handle, 0, 0);

wchar_t** pPtr = new wchar_t*[nInput];
if( !pPtr ) return 0;

for(int i=0; i<nInput; i++) pPtr[i] = (wchar_t*)pInputValues[i].c_str();

bool ret = rcalc_execute(Handle, nInput, pPtr);
delete[] pPtr;

return ret;
}
/*---------------------------------------*/

bool RCALC::Execute(int nInput, const std::string* pInputValues)
{
if( Handle <= 0 ) return 0;
if( nInput <= 0 || !pInputValues ) return rcalc_execute(Handle, 0, 0);

char** pPtr = new char*[nInput];
if( !pPtr ) return 0;

for(int i=0; i<nInput; i++) pPtr[i] = (char*)pInputValues[i].c_str();

bool ret = rcalc_execute_a(Handle, nInput, pPtr);
delete[] pPtr;

return ret;
}
/*---------------------------------------*/

int RCALC::InputCount()
{
if( Handle <= 0 ) return 0;
return rcalc_get_input_count(Handle);
}
/*---------------------------------------*/

int RCALC::InputType(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_input_type(Handle, number);
}
/*---------------------------------------*/

int RCALC::InputTypeNumericScale(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_input_type_numscale(Handle, number);
}
/*---------------------------------------*/

std::wstring RCALC::InputTypeName(int number)
{
std::wstring s;
if( Handle <= 0 || number <= 0 ) return s;

const wchar_t* p = rcalc_get_input_type_name(Handle, number);
if( p ) s = std::wstring(p);

return s;
}
/*---------------------------------------*/

std::string RCALC::InputTypeNameA(int number)
{
std::string s;
if( Handle <= 0 || number <= 0 ) return s;

const char* p = rcalc_get_input_type_name_a(Handle, number);
if( p ) s = std::string(p);

return s;
}
/*---------------------------------------*/

std::wstring RCALC::InputName(int number)
{
std::wstring s;

if( Handle <= 0 || number <= 0 ) return s;
const wchar_t* p = rcalc_get_input_name(Handle, number);
if( p ) s = std::wstring(p);

return s;
}
/*---------------------------------------*/

std::string RCALC::InputNameA(int number)
{
std::string s;

if( Handle <= 0 || number <= 0 ) return s;
const char* p = rcalc_get_input_name_a(Handle, number);
if( p ) s = std::string(p);

return s;
}
/*---------------------------------------*/

bool RCALC::InputIsNull(int number)
{
if( Handle <= 0 || number <= 0 ) return 1;
return rcalc_isnull_input(Handle, number);
}
/*---------------------------------------*/

std::wstring RCALC::InputString(int number)
{
std::wstring s;

if( Handle <= 0 || number <= 0 ) return s;
const wchar_t* p = rcalc_get_input_string(Handle, number);
if( p ) s = std::wstring(p);

return s;
}
/*---------------------------------------*/

std::string RCALC::InputStringA(int number)
{
std::string s;

if( Handle <= 0 || number <= 0 ) return s;
const char* p = rcalc_get_input_string_a(Handle, number);
if( p ) s = std::string(p);

return s;
}
/*---------------------------------------*/

double RCALC::InputDouble(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_input_double(Handle, number);
}
/*---------------------------------------*/

int RCALC::InputInt(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_input_int(Handle, number);
}
/*---------------------------------------*/

__int64 RCALC::InputNumeric(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_input_numeric(Handle, number);
}
/*---------------------------------------*/

__int64 RCALC::InputTime(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_input_time(Handle, number);
}
/*---------------------------------------*/

int RCALC::OutCount()
{
if( Handle <= 0 ) return 0;
return rcalc_get_output_count(Handle);
}
/*---------------------------------------*/

std::wstring RCALC::CompiledText(bool bCRLF)
{
std::wstring s;
if( Handle <= 0 ) return s;

const wchar_t* p = rcalc_get_compiled_text(Handle, bCRLF);
if( p ) s = std::wstring(p);

return s;
}
/*---------------------------------------*/

std::string RCALC::CompiledTextA(bool bCRLF)
{
std::string s;
if( Handle <= 0 ) return s;

const char* p = rcalc_get_compiled_text_a(Handle, bCRLF);
if( p ) s = std::string(p);

return s;
}
/*---------------------------------------*/

bool RCALC::OutIsNull(int number)
{
if( Handle <= 0 || number <= 0 ) return 1;
return rcalc_isnull_output(Handle, number);
}
/*---------------------------------------*/

int RCALC::OutType(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_output_type(Handle, number);
}
/*---------------------------------------*/

int RCALC::OutTypeTextLen(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_output_type_textlen(Handle, number);
}
/*---------------------------------------*/

int RCALC::OutTypeNumericScale(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_output_type_numscale(Handle, number);
}
/*---------------------------------------*/

std::wstring RCALC::OutTypeName(int number)
{
std::wstring s;
if( Handle <= 0 || number <= 0 ) return s;

const wchar_t* p = rcalc_get_output_type_name(Handle, number);
if( p )
  {
   int textlen = rcalc_get_output_type_textlen(Handle, number);
   if( textlen )
     {
      wchar_t buf[256];
      wsprintfW(buf, L"%s(%d)", p, textlen);
      s = std::wstring(buf);
     }
   else s = std::wstring(p);
  }

return s;
}
/*---------------------------------------*/

std::string RCALC::OutTypeNameA(int number)
{
std::string s;
if( Handle <= 0 || number <= 0 ) return s;

const char* p = rcalc_get_output_type_name_a(Handle, number);
if( p )
  {
   int textlen = rcalc_get_output_type_textlen(Handle, number);
   if( textlen )
     {
      char buf[256];
      wsprintfA(buf, "%s(%d)", p, textlen);
      s = std::string(buf);
     }
   else s = std::string(p);
  }

return s;
}
/*---------------------------------------*/

std::wstring RCALC::OutName(int number)
{
std::wstring s;

if( Handle <= 0 || number <= 0 ) return s;
const wchar_t* p = rcalc_get_output_name(Handle, number);
if( p ) s = std::wstring(p);

return s;
}
/*---------------------------------------*/

std::string RCALC::OutNameA(int number)
{
std::string s;

if( Handle <= 0 || number <= 0 ) return s;
const char* p = rcalc_get_output_name_a(Handle, number);
if( p ) s = std::string(p);

return s;
}
/*---------------------------------------*/

std::wstring RCALC::OutString(int number)
{
std::wstring s;

if( Handle <= 0 || number <= 0 ) return s;
const wchar_t* p = rcalc_get_output_string(Handle, number);
if( p ) s = std::wstring(p);

return s;
}
/*---------------------------------------*/

std::string RCALC::OutStringA(int number)
{
std::string s;

if( Handle <= 0 || number <= 0 ) return s;
const char* p = rcalc_get_output_string_a(Handle, number);
if( p ) s = std::string(p);

return s;
}
/*---------------------------------------*/

double RCALC::OutDouble(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_output_double(Handle, number);
}
/*---------------------------------------*/

int RCALC::OutInt(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_output_int(Handle, number);
}
/*---------------------------------------*/

__int64 RCALC::OutNumeric(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_output_numeric(Handle, number);
}
/*---------------------------------------*/

__int64 RCALC::OutTime(int number)
{
if( Handle <= 0 || number <= 0 ) return 0;
return rcalc_get_output_time(Handle, number);
}
/*---------------------------------------*/

bool RCALC::LastError()
{
return ( rcalc_get_error_text_len() > 0 );
}
/*---------------------------------------*/

void RCALC::ShowError(HWND hParent)
{
rcalc_show_error(hParent);
}
/*---------------------------------------*/

std::wstring RCALC::ErrorString()
{
std::wstring s;
int n = rcalc_get_error_text_len();
if( !n ) return s;

wchar_t* buf = (wchar_t*)malloc((n+1)*sizeof(wchar_t));
if( !buf ) return s;

BOOL ret = rcalc_copy_error_text(buf, n+1);
if( !ret ) { free(buf); return s; }

s = std::wstring(buf);
free(buf);
return s;
}
/*---------------------------------------*/

std::string RCALC::ErrorStringA()
{
std::string s;
int n = rcalc_get_error_text_len();
if( !n ) return s;

char* buf = (char*)malloc((n+1)*sizeof(char));
if( !buf ) return s;

BOOL ret = rcalc_copy_error_text_a(buf, n+1);
if( !ret ) { free(buf); return s; }

s = std::string(buf);
free(buf);
return s;
}

