
/*
compile VC++: cl /EHsc test2.cpp rcalc_class.cpp repcoder.c user32.lib
compile BCC32: bcc32 rcalc_test2.cpp rcalc_class.cpp repcoder.c
*/

#include "rcalc_class.h"
#include <iostream>
//------------------------

static void _Execute(RCALC& rc, const wchar_t* InputValues=0, const wchar_t* Sep=0)
{
// Execute RCALC object with some InputValues
bool ok = rc.Execute(InputValues, Sep);
if( !ok )
  {
   RCALC::ShowError(); // Show error in MessageBox

   // Print last error (in the current thread) to Console
   if( RCALC::LastError() ) std::cout<< RCALC::ErrorStringA() << "\n\n";
  }

int nOut = rc.OutCount();
for(int i=1; i<=nOut; i++)
  {
   bool bNull = rc.OutIsNull(i); // Check for null result

   std::string name = rc.OutNameA(i);
   std::string type_name = rc.OutTypeNameA(i);

   std::string result;
   if( bNull ) result = "<NULL>";
   else result = rc.OutStringA(i); // Get the result as std::string (ansi version)

   std::cout<< i << "). " << name << " [" << type_name << "]: " << result << "\n";
  }
}
//------------------------

int main()
{
// Load repcoder.dll
if( !repcoder_load() ) return 0;

// Init repcoder
if( !repc_init() ) { repcoder_free(); return 0; }
//------------------

// Create RCALC object
RCALC rc;

// Compile RCALC script file
bool ok = rc.CompileFile(L".\\Samples\\test2.txt");
if( !ok )
  {
   RCALC::ShowError();
   if( RCALC::LastError() ) std::cout<< RCALC::ErrorStringA() << "\n";

   repc_exit();
   repcoder_free();
   return 0;
  }

// Show compiled text
MessageBoxW(0, rc.CompiledText().c_str(), L"Compiled text", MB_OK);

// Get and print the number of output values
int nOut = rc.OutCount();
std::cout<< "nOut = " << nOut << "\n";

// Get and print the output names and their types
for(int i=1; i<=nOut; i++)
  {
   std::string name = rc.OutNameA(i);
   std::string type_name = rc.OutTypeNameA(i);
   std::cout<< i << "). " << name << " [" << type_name << "]\n";
  }

std::cout << "\nResults:\n\n";

// Execute the script 6 times with different sets of input parameters
_Execute(rc, L"-12.3456###-5.67"); std::cout << "\n"; // standard separator string: "###"

RCALC rc2(rc); // make a copy of rc (will be used at the end)

_Execute(rc, L"0.23|8.43", L"|"); std::cout << "\n"; // custom separator string: "|"
_Execute(rc, L"45 @ -34.67", L" @ "); std::cout << "\n"; // custom separator string: " @ "
_Execute(rc, L"###457.93"); std::cout << "\n"; // NULL value passed as parameter 1
_Execute(rc, L"786###"); std::cout << "\n"; // NULL value passed as parameter 2
_Execute(rc); std::cout << "\n"; // NULL values passed for all input parameters

// Show some results for rc2 (copy of rc after the first execute)
std::cout << "\n";
std::cout << "Out1 (numeric) = " << rc2.OutNumeric(1) << "\n"; // convert to long (64-bit integer)
std::cout << "Out1 (double) = " << rc2.OutDouble(1) << "\n"; // convert to double
std::cout << "Out1 (int) = " << rc2.OutInt(1) << "\n"; // convert to int
std::cout << "Out1 (string) = " << rc2.OutStringA(1) << "\n"; // show as string

std::cout << "\n";
std::cout << "Out2 (numeric) = " << rc2.OutNumeric(2) << "\n";
std::cout << "Out2 (double) = " << rc2.OutDouble(2) << "\n";
std::cout << "Out2 (int) = " << rc2.OutInt(2) << "\n";
std::cout << "Out2 (string) = " << rc2.OutStringA(2) << "\n";

std::cout << "\n";
std::cout << "Out3 (numeric) = " << rc2.OutNumeric(3) << "\n";
std::cout << "Out3 (double) = " << rc2.OutDouble(3) << "\n";
std::cout << "Out3 (int) = " << rc2.OutInt(3) << "\n";
std::cout << "Out3 (string) = " << rc2.OutStringA(3) << "\n";

__int64 tm = rc2.OutTime(8); // date, time and timestamp are returned as long (64-bit)
std::cout << "\n";
std::cout << "Out8 (date) = " << RCALC::Year(tm) << "-" << RCALC::Month(tm) << "-" << RCALC::Day(tm) << "\n"; // extract date parameters from this long result
std::cout << "Out8 (string) = " << rc2.OutStringA(8) << "\n";

// Exit repcoder
repc_exit();

// Free repcoder.dll
repcoder_free();
return 0;
}

