
/*
compile VC++: cl /EHsc thrtest1.cpp repcoder.c user32.lib
compile BCC32: bcc32 -tWM thrtest.cpp repcoder.c
*/

#include "repcoder.h"
#include <process.h>
#include <stdio.h>
//------------------------

static const int NUM_THREADS = 10; // number of threads
static const int N_LOOP = 50000; // number of WHILE loops

// Input structure for a thread
struct THR_STRUCT
{
 int Nr;
 int CalcHandle;
 volatile int bEnd;
};

// Thread procedure
static void _ThrProc(void* ptr)
{
if( !ptr ) return;
THR_STRUCT* pts = (THR_STRUCT*)ptr;

// Make a copy of the primary (compiled) handle
int h = rcalc_copy_handle(pts->CalcHandle);
if( !h ) { rcalc_show_error(0); pts->bEnd = 1; return; }

// Set the 2 input values for the RCALC script
wchar_t InputValues[100];
wsprintfW(InputValues, L"%d###%d", pts->Nr, N_LOOP);

// Execute the RCALC code
rcalc_execute2(h, InputValues, 0);

// Free the copied handle
rcalc_free_handle(h);

// Show RCALC error (if any)
rcalc_show_error(0);

// Mark the volatile flag that the thread has finished
// for the join code in the main() function
pts->bEnd = 1;
}
//------------------------

int main()
{
// Load and init repcoder.dll
if( !repcoder_load() ) return 0;
if( !repc_init() ) { repcoder_free(); return 0; }

// Alloc the primary RCALC handle in the main thread
int h = rcalc_alloc_handle();
if( !h ) { repc_exit(); repcoder_free(); return 0; }

// Compile the handle (only once !!!)
BOOL ok = rcalc_compile_file(h, L".\\Samples\\threads.txt", 0);
if( !ok )
  {
   rcalc_show_error(0);

   rcalc_free_handle(h);
   repc_exit();
   repcoder_free();
   return 0;
  }

printf("\nStarting 11 threads.\nPlease wait ...\n");

// Allocate input structures for child threads
int nThr = NUM_THREADS;
THR_STRUCT* params = new THR_STRUCT[nThr];
if( !params )
  {
   rcalc_free_handle(h);
   repc_exit();
   repcoder_free();
   return 0;
  }

// Start the child threads
for(int i=0; i<nThr; i++)
  {
   params[i].Nr = i+1;
   params[i].CalcHandle = h;
   params[i].bEnd = 0;
   _beginthread(_ThrProc, 0, params+i);
  }

// Execute the RCALC code also in the main thread:
// ... make also a copy of the primary (compiled) handle
int h2 = rcalc_copy_handle(h);
// ... set the 2 input values for the RCALC script
wchar_t InputValues[100];
wsprintfW(InputValues, L"%d###%d", nThr+1, N_LOOP);
// Execute the RCALC code
rcalc_execute2(h2, InputValues, 0);
// Free the copied handle
rcalc_free_handle(h2);

// Join the threads
while(1)
  {
   int bStop = 1;
   for(int i=0; i<nThr; i++)
     {
      if( params[i].bEnd == 0 ) { bStop = 0; break; }
     }
   if( bStop ) break;
   Sleep(100);
  }

printf("\nThe result files were created in the current dir:\n"
"res_1.txt, res_2.txt, ..., res_11.txt\n");

// Delete the input structures
delete[] params;

// Free the primary handle
rcalc_free_handle(h);

// Show RCALC error (if any)
rcalc_show_error(0);

// Exit and free repcoder.dll
repc_exit();
repcoder_free();
return 0;
}

